
#include <stddef.h>
#include <math.h>

#include <avr/io.h>

void uart_init(uint32_t baudrate)
{
  uint16_t baud = (F_CPU / 8 / baudrate - 1) / 2;
  UBRR0L = baud;
  UCSR0B |= (_BV(RXEN0) | _BV(TXEN0));
}

void uart_write(uint8_t c)
{
  while(!(UCSR0A & _BV(UDRE0)))
    ;
  UDR0 = c;
}

uint8_t uart_read(void)
{
  while(!(UCSR0A & _BV(RXC0)))
    ;
  return UDR0;
}

void uart_print(const char* str)
{
  while(*str)
  {
    uart_write(*str);
    ++str;
  }
}

void uart_puts(const char* str)
{
  uart_print(str);
  uart_write('\r');
  uart_write('\n');
}

void uart_dprint(double val)
{
  uint8_t shift = 0;
  uint8_t digit;

  while(val > 9)
  {
    val /= 10;
    ++shift;
  }

  while(shift)
  {
    digit = floor(val);
    digit %= 10;
    uart_write(digit + '0');
    val *= 10;
    --shift;
  }
  digit = floor(val);
  uart_write((digit % 10) + '0');
  uart_write('.');
  digit = floor(val * 10);
  uart_write((digit % 10) + '0');
  digit = floor(val * 100);
  uart_write((digit % 10) + '0');
  digit = floor(val * 1000);
  uart_write((digit % 10) + '0');
}

char* uart_gets(char* buf, size_t length)
{
  size_t i = 0;
  uint8_t c = 0;
  do {
    c = uart_read();
    buf[i++] = c;
  } while(c != '\n' && i < length - 1);
  buf[i] = 0;
  return buf;
}

